# TODO add to pip, update instructions

# TODO Pocket Casts assumes next episode release time - why? how can we control this?

# I'm not sure that those are still relevant, they're from before I started downloading from youtube:
# TODO podcastindex.org doesn't play, or download
# TODO Mac's podcast takes 30 minutes to start playing (Daniel's report in Discord)

import logging
import sys
import tomli
from urllib.parse import urlparse

from podgen import Podcast, Person, Category, htmlencode
from pathvalidate import sanitize_filename

from podtuber.youtube_parser import YoutubeParser

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('podtuber')

example_config_toml_url = 'https://github.com/ZvikaZ/podtuber/blob/master/config.toml'


def get_parser(url):
    if urlparse(url).netloc == 'www.youtube.com' and urlparse(url).path == '/playlist':
        return YoutubeParser(url)
    else:
        logger.error(f'Unsupported playlist: {url}\n'
                     'Currently only YouTube playlists are supported. You can open an issue, maybe your parser will '
                     'be added.')
        sys.exit()


def create_rss(podcast_config, config):
    parser = get_parser(podcast_config['url'])
    logger.info(f'Handling playlist {parser.get_name()}')

    sanitized_title = sanitize_filename(parser.get_name()).replace(' ', '_')
    rss_filename = f'{sanitized_title}.rss'

    podcast = Podcast()
    podcast.name = parser.get_name()
    podcast.description = parser.get_description()
    podcast.website = parser.get_website()
    podcast.explicit = False  # will be updated if one of the episodes is True
    podcast.image = parser.get_image()
    podcast.authors = parser.get_authors()

    podcast.category = Category(podcast_config.get('category'), podcast_config.get('subcategory'))
    podcast.feed_url = f'{config["general"]["base_url"].strip("/")}/{rss_filename}'
    if podcast_config.get('owner_mail'):
        podcast.owner = Person(parser.get_owner_name(), podcast_config.get('owner_mail'))

    # TODO set automatically (e.g., https://github.com/pytube/pytube/issues/1742)
    podcast.language = podcast_config.get('language')

    for parsed_episode in parser.get_episodes():
        try:
            parsed_episode.check_availability()
        except Exception as err:
            logger.warning(f"Skipping '{parsed_episode.get_title()}' ({parsed_episode.get_link()}) because of: {err}")
        else:
            episode = podcast.add_episode()
            # print(clean_jpg_url(v.thumbnail_url))    #TODO use this for episodes as well?
            episode.title = htmlencode(parsed_episode.get_title())
            episode.summary = htmlencode(parsed_episode.get_summary())
            episode.publication_date = parsed_episode.get_publication_date()
            episode.explicit = parsed_episode.get_explicit()
            if episode.explicit:
                podcast.explicit = True
            episode.media = parsed_episode.get_media(base_url=config["general"]["base_url"],
                                                     series_title=sanitized_title)
            episode.id = parsed_episode.get_id()
            episode.link = parsed_episode.get_link()
            episode.authors = parsed_episode.get_authors()

    podcast.rss_file(rss_filename)
    return rss_filename


def main():
    try:
        with open("config.toml", mode="rb") as fp:
            config = tomli.load(fp)
    except FileNotFoundError:
        logger.error('Missing config.toml file in current directory. You can use '
                     f'{example_config_toml_url} as a reference.')
        sys.exit()
    except Exception as err:
        logger.error(err)
        logger.error(f'Illegal config.toml file. You can use {example_config_toml_url} as a reference.')
        sys.exit()
    for podcast_config in config.get('podcasts'):
        rssfile = create_rss(podcast_config, config)
        logger.info(f"Created '{rssfile}'\n")


if __name__ == '__main__':
    main()



import logging
from pathlib import Path
from urllib.parse import quote
import pytz

from pytube import Playlist
from podgen import Person, Media

from podtuber.utils import clean_jpg_url

logger = logging.getLogger(__name__)


class VideoParser:
    def __init__(self, video):
        self.video = video

    def check_availability(self):
        self.video.check_availability()

        # make sure info is parsed (otherwise description might be None)
        # taken from https://github.com/pytube/pytube/issues/1674
        self.video.bypass_age_gate()

    def get_title(self):
        return self.video.title

    def get_summary(self):
        return self.video.description

    def get_publication_date(self):
        return pytz.utc.localize(self.video.publish_date)  # TODO is it really UTC? always?

    def get_explicit(self):
        return self.video.age_restricted

    def get_media(self, base_url, series_title):
        stream = self.video.streams.get_audio_only()  # returns best mp4 audio stream
        return get_media_from_youtube(base_url, series_title, stream)

    def get_id(self):
        return self.video.watch_url

    def get_link(self):
        return self.video.watch_url

    def get_authors(self):
        return [Person(self.video.author)]


class YoutubeParser:
    def __init__(self, playlist_url):
        self.playlist = Playlist(playlist_url)
        assert self.playlist.videos

    def get_name(self):
        return self.playlist.title

    def get_description(self):
        try:
            return self.playlist.description
        except KeyError:
            return self.get_name()

    def get_website(self):
        return self.playlist.playlist_url

    def get_image(self):
        # TODO make sure it's square, at least 1400x1400
        return clean_jpg_url(self.playlist.videos[0].thumbnail_url)

    def get_authors(self):
        return [Person(self.playlist.owner)]

    def get_owner_name(self):
        return self.playlist.owner

    def get_episodes(self):
        for video in self.playlist.videos:
            yield VideoParser(video)


def get_media_from_youtube(base_url, series_title, stream):
    path = Path('files') / series_title
    path.mkdir(parents=True, exist_ok=True)
    stream.subtype = 'm4a'
    file = Path(stream.download(output_path=path))
    logger.info(file)
    media = Media(
        url=f'{base_url}/{quote((path / file.name).as_posix())}',
        size=stream.filesize,
    )
    media.populate_duration_from(file)
    return media



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['podtuber']

package_data = \
{'': ['*']}

install_requires = \
['pathvalidate>=3.1.0,<4.0.0',
 'podgen>=1.1.0,<2.0.0',
 'pytube>=15.0.0,<16.0.0',
 'tomli>=2.0.1,<3.0.0']

entry_points = \
{'console_scripts': ['podtuber = podtuber.main:main']}

setup_kwargs = {
    'name': 'podtuber',
    'version': '0.1.0',
    'description': '',
    'long_description': "podtuber\n========\n\nSimple Python application to create podcast `.rss` files from YouTube playlists.\n\ninstallation\n------------\nYou might want to start with creating a Python virtual env. For example:\n```shell\nconda create -n podtuber python=3\nconda activate podtuber\n```\n\nand then:\n```shell\npip install git+https://github.com/ZvikaZ/podtuber.git\n```\nyou might need to logout and login, in order for the new `podtuber` command to be updated in `PATH`.\n\nusage\n-----\n- copy the [example config.toml](https://github.com/ZvikaZ/podtuber/blob/master/config.toml) to your working directory, and modify it as needed (it's thoroughly commented).\n- and run: \n```shell\npodtuber\n```\n\nnotes\n-----\n- The .rss file needs to be served from an HTTP(S) server. Running the server is out of the scope of this tool.\n\n- :Also, you might want to periodically update the .rss file (because the playlist might have been updated).\nIt can be achieved for example by using a Cron job to run `podtuber` on regular times.",
    'author': 'Zvika Haramaty',
    'author_email': 'haramaty.zvika@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
